/*
 *  grossesfonctions.h
 *  Calcul
 *
 *  Created by Thomas Jannaud on 02/02/08.
 *  Copyright 2008 __Jannaud's Technical Company__. All rights reserved.
 *
 */

#include <stdio.h>
#include <map>
#include <string>
#include "arbre.h"
#include "noeud.h"
#include "Variable.h"
#include "grossesfonctions.h"

using namespace std;

grossesfonctions::grossesfonctions(arbre * a, char ope, char j, arbre * d, arbre * f, arbre * p) : noeud(a, 0), debut(d), fin(f), pas(p), vari(j) {
	this->operateur = ((ope == 'Y') ? somme : produit); // produit : Z
}

grossesfonctions::grossesfonctions(const grossesfonctions & rhs) : noeud(rhs), operateur(rhs.operateur), vari(rhs.vari) {
debut = rhs.debut->clone();
fin = rhs.fin->clone();
pas = rhs.pas->clone();
}

arbre* grossesfonctions::clone() {
return new grossesfonctions(this->fils_g->clone(), this->operateur, this->vari, debut->clone(), fin->clone(), pas->clone());
//return new grossesfonctions(*this);
}

double grossesfonctions::evalue(std::map<char,double> table) {
	int d = (int) (debut->evalue(table));
	int f = (int) (fin->evalue(table));
	int pa = (int) (pas->evalue(table));
	
	if (this->operateur == somme) {
		double s = 0.0;
		for(int i = d; i <= f; i += pa) {
			table[this->vari] = (double) (i);
			s += this->fils_g->evalue(table);
		}
		table.erase(this->vari);
		return s;
	};
	
	double p = 1.0;
	for(int i = d; i <= f; i += pa) {
		table[this->vari] = (double) (i);
		p *= this->fils_g->evalue(table);
	}
	table.erase(this->vari);
	return p;
}

string grossesfonctions::to_string() {
	string s = ((operateur == somme) ? "(∑" : "(∏");
	return (s + fils_g->to_string() + ", " + string(1, vari) + "=" + debut->to_string() + ":" + fin->to_string() + ":" + pas->to_string() + ")");
}

bool grossesfonctions::occurence(char x) {
	return (debut->occurence(x) or fin->occurence(x) or pas->occurence(x));
}

arbre * grossesfonctions::deriver(char x) {
	if (!this->fils_g->occurence(x))
		return new Variable(0.0);
	return new grossesfonctions(this->fils_g->deriver(x), this->operateur, this->vari, this->debut->clone(), this->fin->clone(), this->pas->clone());
}

arbre * grossesfonctions::reduire() {
	try{
	map<char, double> tableauvide;
	fils_g->evalue(tableauvide);
	debut->evalue(tableauvide);
	fin->evalue(tableauvide);
	pas->evalue(tableauvide);
	return new Variable(this->evalue(tableauvide));
	}
	catch (...) {	};

	return new grossesfonctions(this->fils_g->reduire(), this->operateur, this->vari, this->debut->reduire(), this->fin->reduire(), this->pas->reduire());
}