/*
 *  operateurbinaire.cpp
 *  calcul
 *
 *  Created by Thomas Jannaud on 30/01/08.
 *  Copyright 2008 __Jannaud's Technical Company__. All rights reserved.
 *
 */

#include <stdio.h>
#include <map>
#include <cmath>
#include <string>
#include "arbre.h"
#include "noeud.h"
#include "Variable.h"
#include "operateurunaire.h"
#include "operateurbinaire.h"


using namespace std;

operateurbinaire::operateurbinaire(arbre * a, arbre * b, char signe) : noeud(a, b) {
	this->operateur = signe;
}

operateurbinaire::operateurbinaire(const operateurbinaire & rhs) : noeud(rhs), operateur(rhs.operateur) {
}

arbre* operateurbinaire::clone() {
return new operateurbinaire(this->fils_g->clone(), this->fils_d->clone(), this->operateur);
//	return new operateurbinaire(*this);
}

double operateurbinaire::evalue(std::map<char,double> table) {
	double a = this->fils_g->evalue(table), b = fils_d->evalue(table);
	switch (this->operateur) {
		case '+':
			return a+b;
		case '_':
			return a-b;
		case '*':
			return a*b;
		case '/':
			if (b == 0.) 
				throw 2;
			else
				return a/b;
			break;
		case '^':
			return exp(b * log(a));
		default:
			throw 3;
			break;
	}
}	

string operateurbinaire::to_string() {
	string s;
	if (this->operateur == '+' or this->operateur == '-')
		s = "(";
	s += fils_g->to_string() + string(1, this->operateur) + fils_d->to_string();
	if (this->operateur == '+' or this->operateur == '-')
		s += ")";
	return s;
}	

bool operateurbinaire::occurence(char x) {
	return (fils_d->occurence(x) or fils_g->occurence(x));
}

arbre * operateurbinaire::deriver(char x) {
	if (!this->occurence(x))
		return new Variable(0.0);
	
	switch (this->operateur) {
		case '+':
			if (this->fils_g->occurence(x) && this->fils_d->occurence(x))
				return new operateurbinaire(this->fils_g->deriver(x), this->fils_d->deriver(x), '+');
			else if (this->fils_g->occurence(x))
				return this->fils_g->deriver(x);
			else
				return this->fils_d->deriver(x);
			
			case '-':
			if (this->fils_g->occurence(x) && this->fils_d->occurence(x))
				return new operateurbinaire(this->fils_g->deriver(x), this->fils_d->deriver(x), '_');
			else if (this->fils_g->occurence(x))
				return this->fils_g->deriver(x);
			else
				return new operateurbinaire(new Variable(-1.0), this->fils_d->deriver(x), '*');
			
			case '*':
			if (this->fils_g->occurence(x) && this->fils_d->occurence(x)) {
				arbre * a1 = new operateurbinaire(this->fils_g->deriver(x), this->fils_d->clone(), '*');
				arbre * a2 = new operateurbinaire(this->fils_g->clone(), this->fils_d->deriver(x), '*');
				return new operateurbinaire(a1, a2, '+');
			}
			else if (this->fils_g->occurence(x))
				return new operateurbinaire(this->fils_g->deriver(x), this->fils_d->clone(), '*');
			else
				return new operateurbinaire(this->fils_g->clone(), this->fils_d->deriver(x), '*');
			
			case '/':
			if (this->fils_d->occurence(x) && this->fils_g->occurence(x)) {
				arbre * a1 = new operateurbinaire(this->fils_g->deriver(x), this->fils_d->clone(), '/');
				arbre * a2 = new operateurbinaire(this->fils_g->clone(), this->fils_d->deriver(x), '*');
				arbre * a3 = new operateurbinaire(this->fils_d->clone(), this->fils_d->clone(), '*');
				return new operateurbinaire(a1, new operateurbinaire(a2, a3, '/'), '_');
			}			
			else if (this->fils_d->occurence(x)) {
				arbre * a1 = new operateurbinaire(this->fils_g->clone(), this->fils_d->deriver(x), '*');
				arbre * a2 = new operateurbinaire(this->fils_d->clone(), this->fils_d->clone(), '*');
				return new operateurbinaire(new Variable(-1.0), new operateurbinaire(a1, a2, '/'), '*');
			}
			else
				return new operateurbinaire(this->fils_g->deriver(x), this->fils_d->clone(), '/');
			
			case '^': {
				arbre * b0 = new operateurunaire(this->fils_d->clone(), 'L');
				arbre * b1 = new operateurbinaire(this->fils_g->deriver(x), b0, '*');
				arbre * b2 = new operateurbinaire(this->fils_d->deriver(x), this->fils_d->clone(), '/');
				arbre * b3 = new operateurbinaire(b2, this->fils_g->clone(), '*');
				arbre * b4 = new operateurbinaire(b1, b3, '+');
				return new operateurbinaire(b4, this->clone(), '*');
			}
			default:
			return 0;
	}
}

arbre* operateurbinaire::reduire() {
	arbre * a = this->fils_g->reduire();
	arbre * b = this->fils_d->reduire();

	try{
	map<char, double> tableauvide;
	a->evalue(tableauvide);
	b->evalue(tableauvide);
	delete a;
	delete b;
	return new Variable(this->evalue(tableauvide));
	}
	catch (...) {	};
	
	switch (this->operateur) {
		case '+':
			if (a->isvariable(0.0)) {
				delete a;
			return b;}
			else if (b->isvariable(0.0)) {
				delete b;
			return a;}
			else
				return new operateurbinaire(a, b, '+');
			
			case '_':
			if (a->isvariable(0.0)) {
				delete a;
			return new operateurbinaire(new Variable(-1.0), b, '*');}
			else if (b->isvariable(0.0)) {
				delete b;
			return a;}
			else
				return new operateurbinaire(a, b, '_');
			
			case '*':
			if (a->isvariable(0.0) or b->isvariable(0.0)) {
				delete a;
				delete b;
				return new Variable(0.0);
			}
			else if (a->isvariable(1.0)) {
				delete a;
				return b;
			}
			else if (b->isvariable(1.0)) {
				delete b;
				return a;
			}
			else
				return new operateurbinaire(a, b, '*');
			
			case '/':
			if (a->isvariable(0.0)) {
				delete a;
				delete b;
				return new Variable(0.0);
			}
			else if (b->isvariable(1.0)) {
				delete b;
				return a;
			}
			else
				return new operateurbinaire(a, b, '/');
			
			case '^':
			if (b->isvariable(0.0)) {
				delete a;
				delete b;
				return new Variable(1.0);
			}
			else if (a->isvariable(1.0)) {
				delete a;
				delete b;
				return new Variable(1.0);
			}
			else if (a->isvariable(0.0)) {
				delete a;
				delete b;
				return new Variable(0.0);
			}
			else
				return new operateurbinaire(a, b, '^');
			default:
			throw 3;
			break;
	}
}